﻿// Fields
// ------------------------------------------------------------------------------------------

var DialogResult = { "Yes":0, "No":1, "Ok":3, "Cancel":4 };
var BugTrackerMode = { "Single":0, "Multi":1, "Disabled":2 };
var Command = { "Next":0, "Previous":1, "Direct":2, "Update":3, "Close":4, "Finish":5, "Store":6, "Redirect":7, "ShowFinishDialog":8, "Alert":9, "Load":10, "Initialize":11, "KeepAlive":12 };

var PromotedAttributeCollection = new Array();
var NextCommandStack = new Array();

var IsReadOnly = false;
var AllowMultipleBugsPerTestCase = false;
var IsBugTrackerEnabled = false;
var UseBugTrackerItemDialog = false;
var BugTrackerUrl = null;
var BugTrackerProjectUrl = null;
var BugTrackerIssueUrl = null;

var KeepAliveInterval = 0;

var CurrentVariantID = 0;
var LoadedAttachmentTab = false;
var LoadedBugTab = false;

// Initialize
// ------------------------------------------------------------------------------------------

window.onload = function() {

    ResizeContent();
    HandleCommand(Command.Initialize);
};

window.onbeforeunload = function () { return "Translate(SR_JS_MsgUnsavedData)"; };

// Das hier blockiert die Memo-Eingabe im Markdown-Editor.
window.onkeypress = function(e) { return e.keyCode !== 13; };

// Procedures & Logic
// ------------------------------------------------------------------------------------------

function OnSplitterInit(s, e) {
    ResizeContent();
}

function OnSplitterPaneResized(s, e) {
    ResizeContent();
}

function ResizeContent() {

    var left = ContentSplitter.GetPaneByName("ContentPaneLeft");
    var right = ContentSplitter.GetPaneByName("ContentPaneRight");

    ResizeLeftPaneContent(left.GetClientWidth(), left.GetClientHeight());
    ResizeRightPaneContent(right.GetClientWidth(), right.GetClientHeight());

    if ( LockedItemPopup.IsWindowVisible( LockedItemPopup.GetWindow(0) )){
        LockedItemPopup.UpdatePosition();
    }
}

function ResizeLeftPaneContent(width, height) {

    // 2011-07-18 Markus Klieber
    // Also subtract client height of AttributesControl
    var maintableRow1Height = $("#maintableRow1").height();
    var maintableRow2Height = $("#maintableRow2").height();
    var maintableRow3Height = $("#maintableRow3").height();
    var maintableRow4Height = $("#maintableRow4").height();

    $("#maintableRow3").height(
        height - (maintableRow1Height + maintableRow2Height + maintableRow4Height));

    $("#MainLeftContentOuter").height(height);
    $("#MainLeftContentOuter").width(width);

    TestResultHtmlEditor.SetWidth(width - 32);
    TestResultHtmlEditor.SetHeight(height - (95 + maintableRow2Height + maintableRow4Height));

    TestResultAttachmentsContent.SetWidth(width - 32);
    TestResultAttachmentsContent.SetHeight(height - (95 + maintableRow2Height + maintableRow4Height));

    if (isDevExControl("MultiBugContent")) {

        MultiBugContent.SetWidth(width - 32);
        MultiBugContent.SetHeight(height - (95 + maintableRow2Height + maintableRow4Height));
    }

    // Show on init
    // --
    MainLeftTabControl.AdjustControl();
    MainLeftTabControl.AdjustSize();
    MainLeftTabControl.SetVisible(true);
}

function ResizeRightPaneContent(width, height) {
    var title = $("#titlecompact");

    $("#testcase-content-div").height(height);
    $("#contentcompact").height(height - title.height() - 35);
}

function OnTestResultComboBoxSelectedIndexChanged(s, e) {
    e.processOnServer = false;
    HandleCommand( Command.Update );
}

function OnDirectJumpComboBoxInit(s, e) {
    CurrentVariantID = s.GetValue();
}

function OnDirectJumpComboBoxSelectedIndexChanged(s, e) {
    e.processOnServer = false;

    if ( IsValid() ) {
        CheckHandleBugTracker(Command.Direct);
    }
    else {
        // Revert index change
        DirectJumpComboBox.SetValue(CurrentVariantID);
        DirectJumpComboBox.SetCaretPosition(0);
    }
}

function OnNextButtonClick(s, e) {
    e.processOnServer = false;
    CheckHandleBugTracker(Command.Next);
}

function OnPreviousButtonClick(s, e) {
    e.processOnServer = false;
    CheckHandleBugTracker(Command.Previous);
}

function OnCloseButtonClick(s, e) {
    e.processOnServer = false;
    CheckHandleBugTracker(Command.Close);
}

function HandleCommand(command){

    if (command == Command.Initialize || command == Command.Update || command == Command.KeepAlive ) {

        ProcessRequest(command, false);
    }
    else {

        if ( IsValid() ) {
            ProcessRequest(command, true);
        }
    }
}

function CheckHandleNextCommand() {

    if (NextCommandStack.length > 0) {
        var next = NextCommandStack.pop();

        HandleCommand( next );
    }
}

function getResultHtml() {
    try {
        if (typeof TestResultHtmlEditor.GetHtml === 'function') {
            return TestResultHtmlEditor.GetHtml();
        } else {
            // Wenn Markdown-Modus.
            return TestResultHtmlEditor.GetText();
        }
    } catch (err) {
        // Wenn Markdown-Modus.
        return TestResultHtmlEditor.GetText();
    }
}

function setResultHtml(html) {
    try {
        if (typeof TestResultHtmlEditor.SetHtml === 'function') {
            TestResultHtmlEditor.SetHtml(html);
        } else {
            // Wenn Markdown-Modus.
            TestResultHtmlEditor.SetText(html);
        }
    } catch (err) {
        // Wenn Markdown-Modus.
        TestResultHtmlEditor.SetText(html);
    }
}

function CheckHandleBugTracker( command ) {

    if ( !IsValid() ) {
        return;
    }

    if (IsBugTrackerEnabled) {

        if (AllowMultipleBugsPerTestCase) {
            // ---
        }
        else {
            if ( AddToBugTrackerCheckBox.GetChecked() ) {

                var issueId = parseInt( BugTrackerIssueIDTextBox.GetText() );

                if ( isNaN(issueId) ) {
                    AddBugTrackerItem(getResultHtml(), UseBugTrackerItemDialog);
                }
                else {
                    UpdateBugTrackerItem(issueId, getResultHtml(), UseBugTrackerItemDialog);
                }

                NextCommandStack.push(command);

                return;
            }
        }
    }

    HandleCommand(command);
}

function OnConfirmFinishTestRunDialogClosed(s, e) {
    if (s.DialogResult == DialogResult.Yes) {
        HandleCommand( Command.Finish );
    }
    if (s.DialogResult == DialogResult.No) {
        RedirectOrClose();
    }
}

function OnEditBugTrackerItemDialogClosed(s, e) {

    if (s.DialogResult == DialogResult.Ok) {
        if (AllowMultipleBugsPerTestCase) {
            ReloadBugs();
        }
    }
    CheckHandleNextCommand();
}

function OnMainLeftTabControlActiveTabChanged(s, e) {

    if (e.tab.name == "Attachments" && !LoadedAttachmentTab) {

        ReloadAttachments();

        LoadedAttachmentTab = true;
    }

    if (e.tab.name == "Bugs" && !LoadedBugTab) {

        ReloadBugs();

        LoadedBugTab = true;
    }
}

function ProcessRequest(command, showLoadingPanel) {

    if (showLoadingPanel) {
        MainLoadingPanel.Show();
    }

    var request = new Object();
    request.command = command;

    if ( NeedStore(command) ) {
        request.data = CoreStore();
    }

    AsyncServerCallback.PerformCallback(
        Sys.Serialization.JavaScriptSerializer.serialize(request));
}

function ProcessResponse(e) {

    if (e.result == null ) {
        // --
    }
    else {

        var result = Sys.Serialization.JavaScriptSerializer.deserialize(e.result);

        if (result.command == Command.Initialize) {

            IsReadOnly = result.data.IsReadOnly;
            UseBugTrackerItemDialog = result.data.UseBugTrackerItemDialog;
            KeepAliveInterval = result.data.KeepAliveInterval;
            BeforeUnloadMessage = result.data.BeforeUnloadMessage;
            BugTrackerUrl = result.data.BugTrackerUrl;
            BugTrackerProjectUrl = result.data.BugTrackerProjectUrl;
            BugTrackerIssueUrl = result.data.BugTrackerIssueUrl;

            if (result.data.BugTrackerMode != BugTrackerMode.Disabled) {

                IsBugTrackerEnabled = true;
                AllowMultipleBugsPerTestCase = (result.data.BugTrackerMode == BugTrackerMode.Multi);
            }

            window.setTimeout(KeepSessionAlive, KeepAliveInterval);

            DelayLoad();
        }
        else if (result.command == Command.Alert) {

            alert(result.message);
        }
        else {

            if (result.command == Command.Load) {

                CoreLoad(result.data);
                DelayLoad();
                ResetState();
            }
            if (result.command == Command.ShowFinishDialog) {
                ConfirmFinishTestRunDialog.Show();
            }
            if (result.command == Command.Redirect) {
                RedirectOrClose();
            }

            UpdateUI(result);
        }
    }

    MainLoadingPanel.Hide();
}

function CoreStore() {

    StorePromotedAttributeValues();

    var data = new Object();

    data.ResultID = TestResultComboBox.GetValue();
    data.VariantID = DirectJumpComboBox.GetValue();
    data.Description = getResultHtml();
    data.PromotedAttributeCollection = PromotedAttributeCollection;

    return data;
}

function CoreLoad(data) {

    MainLeftTabControl.SetActiveTabIndex(0);
    CurrentVariantID = data.VariantID;

    if (data.ResultID <= 0) {
        TestResultComboBox.SetSelectedIndex( 0 );
    }
    else {
        TestResultComboBox.SetValue(data.ResultID);
    }

    setResultHtml(data.Description);
    DirectJumpComboBox.SetValue(data.VariantID);
    DirectJumpComboBox.SetCaretPosition(0);
    TestCasePathLabel.SetText(data.TestCaseTitle);

    if (IsBugTrackerEnabled && !AllowMultipleBugsPerTestCase) {

        BugTrackerIssueUrl = data.BugTrackerIssueUrl;

        if ( data.BugTrackerIssueID == 0 ) {
            BugTrackerIssueIDTextBox.SetText("");
        }
        else {
            BugTrackerIssueIDTextBox.SetText(data.BugTrackerIssueID);
        }
    }

    LoadPromotedAttributeValues( data.PromotedAttributeCollection );
}

function DelayLoad() {

    // Visible parts
    // --
    DisplayTestCaseCallbackPanel.PerformCallback();

    // Non visible
    // --
}

function ResetState() {

    LoadedAttachmentTab = false;
    LoadedBugTab = false;
}

function IsValid() {
    return ASPxClientEdit.ValidateGroup(null, false);
}

function NeedStore(command) {

    if (command == Command.Initialize || command == Command.Finish || command == Command.KeepAlive) {
        return false;
    }

    return true;
}

function StorePromotedAttributeValues() {

    if (PromotedAttributeCollection && PromotedAttributeCollection.length > 0) {

        for (var x = 0; x < PromotedAttributeCollection.length; x++) {

            var control = eval(PromotedAttributeCollection[x].ClientID);

            if (control) {
                if (PromotedAttributeCollection[x].Type == "TextBoxSingleLine") {
                    PromotedAttributeCollection[x].Value = control.GetText();
                }
                if (PromotedAttributeCollection[x].Type == "TextBoxMultiLine") {
                    PromotedAttributeCollection[x].Value = control.GetText();
                }
                if (PromotedAttributeCollection[x].Type == "TextBoxSingleLinePassword") {
                    PromotedAttributeCollection[x].Value = control.GetText();
                }
                if (PromotedAttributeCollection[x].Type == "DatePicker") {
                    PromotedAttributeCollection[x].Value = control.GetDate();
                }
                if (PromotedAttributeCollection[x].Type == "CheckBox") {
                    PromotedAttributeCollection[x].Value = control.GetChecked();
                }
                if (PromotedAttributeCollection[x].Type == "DropDownList") {
                    PromotedAttributeCollection[x].Value = control.GetValue();
                }
            }
        }
    }
}

function LoadPromotedAttributeValues(collection) {

    if (collection) {

        PromotedAttributeCollection = collection;

        for (var x = 0; x < PromotedAttributeCollection.length; x++) {

            var control = eval(PromotedAttributeCollection[x].ClientID);
            var value = PromotedAttributeCollection[x].Value;

            if (control) {
                if (PromotedAttributeCollection[x].Type == "TextBoxSingleLine") {
                    control.SetText(value);
                }
                if (PromotedAttributeCollection[x].Type == "TextBoxMultiLine") {
                    control.SetText(value);
                }
                if (PromotedAttributeCollection[x].Type == "TextBoxSingleLinePassword") {
                    control.SetText(value);
                }
                if (PromotedAttributeCollection[x].Type == "DatePicker") {
                    if (value == null || isNaN( Date.parse(value) ) ) {
                        control.SetDate(new Date());
                    }
                    else {
                        control.SetDate(
                            Date.parse(value) );
                    }
                }
                if (PromotedAttributeCollection[x].Type == "CheckBox") {
                    control.SetChecked(
                        PromotedAttributeCollection[x].Value == "True" ||
                        PromotedAttributeCollection[x].Value == "true");
                }
                if (PromotedAttributeCollection[x].Type == "DropDownList") {
                    control.SetValue(PromotedAttributeCollection[x].Value);
                }
            }
        }
    }
}

function UpdateUI(result) {

    if (result == null || result.data == null) {
        return;
    }

    NextButton.SetEnabled( result.data.HasNextItem );
    PreviousButton.SetEnabled(result.data.HasPreviousItem);

    if (result.data.LockInfo.CanEdit) {
        LockedItemPopup.Hide();
        MainLeftContent.SetVisible(true);
    }
    else {
        LockedItemErrorMessageLabel.SetText(result.data.LockInfo.ErrorMessage);
        LockedItemPopup.Show();
        MainLeftContent.SetVisible(false);
    }

    if (IsBugTrackerEnabled) {

        if (AllowMultipleBugsPerTestCase) {
            // ---
        }
        else {

            AddToBugTrackerCheckBox.SetChecked(
                result.data.AutomaticallyAddToBugTracker );
        }
    }
}

function KeepSessionAlive() {

    HandleCommand( Command.KeepAlive );
    window.setTimeout(KeepSessionAlive, KeepAliveInterval );
}

function RedirectOrClose() {

    window.onbeforeunload = null;

    var backUrl = getQuerystring("BackUrl");

    if (backUrl != null && backUrl.length > 0) {
        redirect(backUrl);
    }
    else {
        window.close();
    }
}

function OnAsyncServerBeginCallback(s, e) {
}

function OnAsyncServerCallbackComplete(s, e) {
    ProcessResponse(e);
}

function OnAsyncServerCallbackError(s, e) {
}

function OnAsyncServerEndCallback(s, e) {
}

function OnAsyncServerCallbackInit(s, e) {
}

function OnDisplayTestCaseCallbackPanelBeginCallback(s, e) {

}

function OnDisplayTestCaseCallbackPanelEndCallback(s, e) {
    ResizeContent();
}

// Helper
// ------------------------------------------------------------------------------------------

function redirect(url) {
    if (url) {
        window.location = url;
    }
}

function getQuerystring(key, defaultValue)
{
    if (defaultValue==null) defaultValue="";
    key = key.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
    var regex = new RegExp("[\\?&]"+key+"=([^&#]*)");
    var qs = regex.exec(window.location.href);
    if(qs == null)
    return defaultValue;
    else
    return qs[1];
}

function isDevExControl(cin)
{
    var control = ASPxClientControl.GetControlCollection().GetByName( cin );

    return !isNullOrUndefined(control);
}

function isNullOrUndefined(object) {
    if (object == null) {
        return true;
    }
    if (typeof(object) == 'undefined') {
        return true;
    }
    return false;
}